require 'aws-sdk-s3'
require 'cht-lib-fabric'
require 'cht-lib-scheduler'
require 'observer'
require 'lz4-ruby'
require 'cht-amazon-pricing'
require 'require_all'
require 'job_management'

require 'cht-svc-gdas/version'

module Gdas

  module Assets
    module Aws
      # Stub out
    end
  end

  module Context
    autoload :Account, 'cht-svc-gdas/context/account'
    autoload :Runtime, 'cht-svc-gdas/context/runtime'
  end

  module Clients
    autoload :Redis, 'cht-svc-gdas/clients/redis'
  end

  module Clouds
    autoload :AwsCloudConfig, 'gdas/clouds/aws'
    autoload :AzureCloudConfig, 'gdas/clouds/azure'
  end

  module Constants
    autoload :Ec2Os, 'cht-svc-gdas/constants/ec2_os'
  end

  module Collection
    autoload :Executor, 'gdas/collection/executor'
    autoload :DerivedCollector, 'gdas/collection/derived_collector'
    autoload :AwsDerivedCollector, 'gdas/collection/aws_derived_collector'
    autoload :AzureDerivedCollector, 'gdas/collection/azure_derived_collector'
    autoload :DerivedAssetCollectionTask, 'gdas/collection/derived_asset_collection_task'
  end

  module Processing
    autoload :Common, 'gdas/processing/common'
    autoload :Tags, 'gdas/processing/tags'
    autoload :EmojiHelper, 'gdas/processing/emoji_helper'
    autoload :DerivedAssetProcessor, 'gdas/processing/derived_asset_processor'
  end

  module Scheduler
    autoload :AccountTask, 'gdas/scheduler/account_task'
    autoload :AccountWatcher, 'gdas/scheduler/account_watcher'
    autoload :BaseWatcher, 'gdas/scheduler/base_watcher'
    autoload :SpendWatcher, 'gdas/scheduler/spend_watcher'
    autoload :SpendMonitor, 'gdas/scheduler/spend_monitor'
    autoload :MetricsAccountWatcher, 'gdas/scheduler/metrics_account_watcher'
    autoload :JobStatsGenScheduler, 'gdas/scheduler/job_stats_gen_scheduler'
    autoload :JobTokenCleanupScheduler, 'gdas/scheduler/job_token_cleanup_scheduler'
  end

  module Helpers
    autoload :ModelTermination, 'cht-svc-gdas/helpers/model_termination'
    autoload :ModelActivityFeed, 'cht-svc-gdas/helpers/model_activity_feed'
    autoload :ModelDefaultScopes, 'cht-svc-gdas/helpers/model_default_scopes'
    autoload :ModelReplication, 'cht-svc-gdas/helpers/model_replication'
    autoload :Ec2Os, 'cht-svc-gdas/helpers/ec2_os'
    autoload :Ec2It, 'cht-svc-gdas/helpers/ec2_it'
    autoload :RdsUtils, 'cht-svc-gdas/helpers/rds_utils'
    autoload :RegionHelper, 'cht-svc-gdas/helpers/region_helper'
  end

  module Workers
    autoload :ProcessingAssetsWorker, 'gdas/workers/processing_assets'
    autoload :AwsProcessingAssetsWorker, 'gdas/workers/processing_assets'
    autoload :AzureProcessingAssetsWorker, 'gdas/workers/processing_assets'
    autoload :AwsGdasWorker, 'gdas/workers/generic_derived_assets'
    autoload :AzureGdasWorker, 'gdas/workers/generic_derived_assets'
  end

  module JobMgmt
    autoload :TokenLock, 'cht-svc-gdas/job_mgmt/token_lock'
    autoload :DerivedProcessorTokenLock, 'cht-svc-gdas/job_mgmt/derived_processor_token_lock'
    autoload :ConfigurableReschedule, 'cht-svc-gdas/job_mgmt/configurable_reschedule'
  end

  module Audit
    autoload :ProcessingAssetsAuditor, 'gdas/audit/processing_assets_auditor'
  end
end
