# This file contains elements of cht_shared/lib/cht_shared/ec2_it.rb
module Gdas
  module Helpers
    module Ec2It
      extend ::ActiveSupport::Concern

      # this regex is used to parse instance type information from an item description
      # $1 will be the instance type (family and size of the instance)
      # #2 will be the size of the instance type
      # $3 will be the Usage Type (family and size of the usage instance)
      # $4 will be the size of the usage type
      # $3 and $4 are nil but the regex was a match, then Usage Type is m1.small
      # we use character class [^ ] instead of "." so an item description like the following doesn't match too early:
      # USD 10.99 hourly fee per Linux/UNIX (Amazon VPC), x1.32xlarge instance; UsageType: EU-BoxUsage:x1.32xlarge
      M1_SMALL = 'm1.small'.freeze

      def get_instance_type_from_ut_and_desc(usage_type, item_desc)
        item_desc =~ /UsageType: .*Usage:([^ ]*)/
        instance_type = Regexp.last_match(1)
        return instance_type unless instance_type.nil?

        usage_type =~ /Usage:([^ ]*)/
        Regexp.last_match(1) || M1_SMALL
      end
    end
  end
end
