module Gdas
  module Helpers
    module ModelTermination
      extend ::ActiveSupport::Concern

      included do
        class << self
          class_attribute :model_terminated_attributes, :model_children

          def terminated_attributes(attrs)
            self.model_terminated_attributes = attrs
            before_save :adjust_termination_status
          end

          def terminate_relations(*relations)
            self.model_children = Array(relations)
            before_save :invoke_relation_termination
          end
        end

        def adjust_termination_status
          return unless is_active_changed?
          # Only if we went from true to false
          return unless changes['is_active'][0] && !changes['is_active'][1]
          self.class.model_terminated_attributes&.each_pair do |k, v|
            cur_val = __send__ k.to_sym
            next unless v[:current].include? cur_val
            __send__ "#{k}=".to_sym, v[:final]
          end
        end

        def invoke_relation_termination
          return if self.class.model_children.blank?
          return unless is_active_changed?
          # Only if we went from true to false
          return unless changes['is_active'][0] && !changes['is_active'][1]
          inactivated_time = ::Time.now
          self.class.model_children.each do |rel|
            # For single-entity relationships (ex: +:has_one+), we can't apply scoping initially because calling the
            # relationship immediately returns the asset, not a scape-able query.
            rel_type = self.class.reflect_on_association(rel).macro # Get relationship type. Ex: +:has_one+
            if [:has_one, :belongs_to].include?(rel_type)
              related_asset = send(rel) # Get the single asset (or nil)
              next unless related_asset # If it had no relationship set, move on.
              # Only continue if there is an active asset to inactivate.
              next unless related_asset.respond_to?(:is_active) && related_asset.is_active
              assets = Array(related_asset)
            else
              # For multiple-entity relationships (ex: +:has_many+), just apply the +active+ scope.
              assets = Array(send(rel).active)
            end

            assets.each do |asset|
              # FUTURE: pass in `audit_log` to track events in kafka
              ::CloudHealth::Processing::Actions::TerminateAsset.call asset: asset,
                                                                      inactivated_at: inactivated_time,
                                                                      account: aws_account
            end
          end
        end
      end
    end
  end
end
