module Gdas::JobMgmt

  # Subclass of TokenLock that implements the "token_owners" method.
  #
  # This method gets the dimensions and values from the job class that
  # we want to set as token owners.
  #

  #

  class TokenLock < ::CloudHealth::JobManagement::Locks::TokenLock
    include Import['logger', 'statsd']

    def initialize(item, callback, redis_pool = nil, statsd_p = nil)
      super(item, callback, redis_pool, statsd_p)
    end

    # The default implementation provides "all known values"
    def self.supported_owners
      DerivedProcessorTokenLock.supported_owners
    end

    # We don't want to just take all argument values, so we'll define the
    # dimensions we're interested in using for ownership
    #
    # Subclasses can override this.
    def supported_owners
      []
    end

    # This makes the client lock not do anything.  Similar to the implementation of the "while_executing" lock.
    def lock
      job_id = item[::SidekiqUniqueJobs::JID]
      yield if block_given?

      job_id
    end

    # This method should return a hash of token type to token owner, compatible
    # with BaseTokenManager's claim_token.
    #
    def token_owners(item)
      # configure stats, if not configured.  Can't do this in the constructor, because `statsd` is defined,
      # but not resolved, at that time.
      @statsd_c ||= statsd
      owners={}
      return nil unless item!=nil && item['args'] && context_from_item(item)
      args = context_from_item(item)
      args.each do |arg, val|
        if supported_owners.include? arg
          owners[arg]=val unless val.nil? || val == ""
        end
      end
      owners.size > 0 ? owners : nil
    end

    def context_from_item(item)
      item['args'][0]
    end

    # Retrieve the unique job ID from the job's context
    def unique_job_id(item)
      return nil unless item!=nil && item['args']
      args = context_from_item(item)
      return nil unless args
      args["unique_job_id"]
    end

    # This class monkey-patches the class in cht-framework-cp to support 7.1.30.
    # We should upgrade all services to use the new gem and then update
    # cht-framework-cp.
    def call_strategy
      super(origin: :server)
    end
  end
end
